/*===========================================================================*\

   $Id$

   Copyright (C) 2007 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

#ifndef PLANARMESH_H
#define PLANARMESH_H

#include <vector>

#include "Vector.h"
#include "Coord.h"
#include "RenderableObject.h"

SPUG_RCPTR(PlanarMesh);
SPUG_RCPTR(Material);

/**
 * A mesh in which all of the points are on the same plane (expected to be
 * used for very large polygons where lighting is an issue - converting to a
 * mesh gives them a greater density of vertices)
 */
class PlanarMesh : public RenderableObject {
   public:
      class Vertex : public Vector, public Coord {
         public:
            Vertex() {}

            Vertex(const Vector &vec, const Coord &coord) : 
               Vector(vec),
               Coord(coord) {
            }

            Vertex operator +(const Vertex &other) const {
               return Vertex(Vector::operator +(other), 
                             Coord::operator +(other)
                             );
            }

            Vertex operator -(const Vertex &other) const {
               return Vertex(Vector::operator -(other),
                             Coord::operator -(other)
                             );
            }

            Vertex operator *(Float val) const {
               return Vertex(Vector::operator *(val),
                             Coord::operator *(val)
                             );
            }
      };

      typedef unsigned VertexIndex;

      struct Quad {
         PlanarMesh *mesh;
         VertexIndex coords[4];
         Quad(VertexIndex a, VertexIndex b, VertexIndex c, VertexIndex d) : 
               mesh(0) {
            coords[0] = a;
            coords[1] = b;
            coords[2] = c;
            coords[3] = d;
         }

         bool operator ==(const Quad &other) const {
            for (int i = 0; i < 4; ++i)
               if (coords[i] != other.coords[i])
                  return false;
            return true;
         }

         const Vertex &operator [](int index) const {
            return mesh->getVertex(coords[index]);
         }

         Vertex &operator [](int index) {
            return mesh->getVertex(coords[index]);
         }
      };

   private:
      std::vector<Vertex> vertices;
      typedef std::vector<Quad> QuadVector;
      QuadVector quads;

      Vector normal;
      MaterialPtr material;

   public:
      PlanarMesh(const Vector &normal, const MaterialPtr &material);

      VertexIndex addVertex(const Vertex &vertex) {
         vertices.push_back(vertex);
         return vertices.size() - 1;
      }

      Vertex &getVertex(VertexIndex index) {
         return vertices[index];
      }

      void addQuad(Quad &quad) {
         quad.mesh = this;
         quads.push_back(quad);
      }

      void removeQuad(const Quad &quad);

      virtual void render(Scene &scene);
};

#endif
