#ifndef QUADOBJ_H
#define QUADOBJ_H

#include <spug/Tracer.h>
#include "PolyObj.h"
#include "Quad.h"
#include "Tri.h"
#include "PlanarMesh.h"

/**
 * A quadrilateral PolyObj.  Defined by four points.
 *
 * This is an immutable object.
 */
class QuadObj : public PolyObj, public Quad {
   private:

      // trace flag to enable tesselation tracing
      static spug::Tracer::Level tesselation;

      // defines the mapping from the vertex to the 2D texture coordinate
      // this is either null or a pointer to an array of four coordinates
      Coord *map;

      // generates the "map" variable based on the positions of the vertices
      void makeMap();

      // this is set to true once we've checked the objects size for lighting
      // considerations.  If it is true, then we've either verified that the
      // object is small enough, or created a PlanarMesh for it.
      bool checkedSize;

      // if the quad is too big for lighting (if any of the edges are greater
      // than the max vertex distance defined by the Graphics singleton) this
      // will hold a mesh representing the quad as smaller sub-quads
      PlanarMeshPtr mesh;

      // generates the mesh object - returns true 
      void makeMesh(const PlanarMesh::Quad &quad);

   public:
      QuadObj(const QuadObj &other);

      QuadObj(const Quad &quad, const MaterialPtr &material) : 
	 Quad(quad), 
	 PolyObj(material),
         map(0),
         checkedSize(false) {
      }

      /**
       * @param map if provided, this is an array of four coordinates.  
       *            Ownership of the array will be transferred to the new
       *            object.
       */ 
      QuadObj(Vector coord0, Vector coord1, Vector coord2, Vector coord3, 
	      const MaterialPtr &material,
              Coord *map = 0
	      ) :
	 Quad(coord0, coord1, coord2, coord3),
	 PolyObj(material),
         map(map),
         checkedSize(false) {
      }

      ~QuadObj();

      // Implements Renderable
      virtual void render(Scene &scene);

      // Implements PolyObj
      virtual Vector normal() const {
	 return Tri::normal(coord0, coord1, coord2);
      }

      // Implements PolyObj
      virtual bool intersects(const Vector &origin, const Vector &direction,
			      Vector &intersection,
			      Float &distance
	                      );

      // Implements PolyObj
      virtual AxisBox getBoundingBox() const;
};

typedef spug::RCPtr<QuadObj> QuadObjPtr;

#endif
