/*===========================================================================*\

   $Id$

   This file describes the python bindings of the program.  It is used by the
   "cppre" program to generate pybind_wurld.cc.

   Copyright (C) 2006 Michael A. Muller

   Permission is granted to use, modify and redistribute this code,
   providing that the following conditions are met:
 
   1) This copyright/licensing notice must remain intact.
   2) If the code is modified and redistributed, the modifications must 
   include documentation indicating that the code has been modified.
   3) The author(s) of this code must be indemnified and held harmless
   against any damage resulting from the use of this code.

   This code comes with ABSOLUTELY NO WARRANTEE, not even the implied 
   warrantee of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

\*===========================================================================*/

module wurld;
include "PythonPluginInternal.h";

/**
 * A key binding.
 */
@python_inst_rcptr
class KeyCommand {}

/**
 * Creates a new key command object. Bind it to a key with KeyHandler::bind().
 */
KeyCommand createKeyCommand(PyObject onKeyPress, PyObject onKeyRelease);

/**
 * A keyboard handler.
 */
@python_inst_pointer
class KeyHandler {
   void bind(int key, KeyCommand command);
}

/**
 * The display driver singleton.  It generalizes the interface to the
 * underlying widget toolkit and window manager.
 */
@python_inst_pointer
class DisplayDriver {
   public:
      void setDisplaySize(int width, int height);

      /**
       * Returns the current key handler for the display.
       */
      KeyHandler getKeyHandler() const;
}

/**
 * Returns the global display driver.
 */
DisplayDriver getDisplayDriver();

/**
 * A game object.  These are added to the game engine for rendering and
 * callback procesing.
 */
@python_inst_rcptr
class GameObject { }

/**
 * A vector is used to represent a set of coordinates in 3D space.
 */
class Vector {
   public:
      /** The X, Y, and Z coordinates */
      /** @{ */
      @attr float x;
      @attr float y;
      @attr float z;
      /** @} */

      /** 
       * Multiplies the vector by the value, effectively changing the
       * magnitude of the vector.
       */
      Vector operator *(float val);

      /**
       * Returns the dot-product of the two vectors.
       */
      Vector operator +(Vector other);
}

/**
 * A rotation matrix is a 3x3 matrix that can be used to rotate a vector.  The
 * cross product of the original vector and the rotation matrix is the rotated
 * vector.
 *
 * To combine multiple rotations, create a cross-product using the
 * makeProduct() method.
 */
class RotMatrix { 
   public:

      /**
       * Rotates "point" and stores the rotated point in "result".
       */
      void rotate(Vector point, Vector result);

      /**
       * Converts the rotation matrix to the cross-product of a and b.
       *
       * This is used to combine multiple rotations - to rotate around
       * absolute axis, use a = current rotation, b = absolute rotation.  To
       * rotate around the axis relative to the current rotation, use a =
       * relative rotation, b = current rotation.
       */
      void makeProduct(RotMatrix a, RotMatrix b);

      /**
       * Converts the matrix into the inverse of the other matrix.  For
       * rotation matrices, the inverse is always equal to the transpose, so
       * in truth, this method calculates the transpose of "other".
       */
      void makeInverse(RotMatrix other);
}

/**
 * Returns the rotation matrix for the given yaw angle (rotation around the Y
 * axis)
 */
RotMatrix createYawRotMatrix(float degrees);

/**
 * Returns the rotation matrix for the given pitch angle (rotation around the
 * X axis)
 */
RotMatrix createPitchRotMatrix(float degrees);

/**
 * Returns the rotation matrix for the given roll angle (rotation around the Z
 * axis.
 */
RotMatrix createRollRotMatrix(float degrees);

/**
 * Creates a vector.
 */
Vector createVector(float x, float y, float z);

/**
 * A texture object.  A texture is just an image used for constructing a
 * material.
 */
@python_inst_rcptr
class Texture { }

/** 
 * Creates a new texture from the given filename - this should be a PNG file 
 */
Texture createTexture(string filename);

/** A Material object.  Materials define the appearences of surfaces. */
@python_inst_rcptr
class Material { }

/** 
 * Creates a new material out of the given texture.
 */
Material createTextureMaterial(Texture texture, float xppus, float yppus);

/** A polygon (abstract class) */
@python_inst_rcptr
class PolyObj { }

/** Creates a quadrilateral */
PolyObj createQuad(Vector c0, Vector c1, Vector c2, Vector c3, 
		   Material material
		   );

/** A model.  Models are just collections of polygons. */
@python_inst_rcptr
class Model {
   public:
      /** Adds a new polygon to the model. */
      void addPoly(PolyObj poly);
}

/** Creates a new, empty model */
Model createModel();

/** Moveables are objects that can have models, position and velocity. */
@python_inst_rcptr
class Moveable {
   public:
      void setModel(Model model);

      /** Get and set the objects position */
      /** @{ */
      void setPosition(Vector position);
      Vector getPosition() const;
      /** @} */

      /** Get and set the object's rotation */
      /** @{ */
      RotMatrix getRotation() const;
      void setRotation(RotMatrix rotation);
      /** @} */

      /** Get and set velocity - velocity is measured in units per second */
      /** @{ */
      void setVelocity(Vector velocity);
      Vector getVelocity();
      /** @} */
}

/** Create a new moveable object.  Position is (0, 0, 0), velocity is zero */
Moveable createMoveable();

/**
 * The camera.  Allows you to control the position and orientation of the
 * camera.
 */
@python_inst_pointer
class Camera {
   public:
      /** Returns the current position of the camera. */
      Vector getOrigin() const;

      /** Sets the current position of the camera. */
      void setOrigin(Vector origin);

      /** Returns the current orientation of the camera */
      RotMatrix getRotation() const;

      /** Sets the current orientaton of the camera */
      void setRotation(RotMatrix rotation);
}

/** 
 * A renderable object.  Normally we use PythonObject (defined below) for
 * game objects, but we still need Renderables (which should be a base class)
 * for HUD stickers.
 */
@python_inst_rcptr
class RenderableObject {}

/**
 * Creates a simple text object for use in the HUD.
 */
RenderableObject createSimpleText(string text, float left, float top, 
                                  float width,
                                  float height
                                  );

/** 
 * A python object wraps a Moveable, allowing you to provide callback into
 * python code for every engine frame.  This is derived from GameObject,
 * although you wouldn't realize it from this prototype.
 */
@python_inst_rcptr
class PythonObject {
   public:
      /**
       * Sets the object to be called every engine frame (10 times per
       * second).  The callable should accept two arguments - the first will
       * be the game engine, the second will be the python object.
       */
      void setEngineFrameCallback(PyObject callback);

      /**
       * Sets the moveable object associated with the PythonObject.
       */
      void setMoveable(Moveable moveable);
}

/** returns a new python object */
PythonObject createPythonObject();

/**
 * The Game Engine.  This object manages all of the activity and rendering.
 */
@python_inst_pointer
class GameEngine {
   public:
      /**
       * Add a new game object
       */
      void addGameObject(GameObject object);

      /**
       * Add a new object to the set of objects that get rendered.
       */
      void addRenderableObject(PythonObject object);

      /**
       * Returns the camera object - which basically defines the position and
       * rotation of the player's view.
       */
      Camera getCameraPtr();

      /**
       * Add a new HUD sticker.
       */
      void addHUDSticker(RenderableObject sticker);

      /**
       * Remove an existing HUD sticker.
       */
      void removeHUDSticker(RenderableObject sticker);
}

GameEngine getGameEngine();

/**
 * Creates a FPS widget for the HUD.
 */
RenderableObject createFPSDisplay(GameEngine engine);

@python_inst_pointer
class Graphics {
   public:
      void enableLighting(int enabled);
}

Graphics getGraphics();
